'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _rawSession = require('./rawSession.js');

var _rawSession2 = _interopRequireDefault(_rawSession);

var _config = require('./config.js');

var _message = require('./message.js');

var _translate = require('./translate.js');

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var log = (0, _debug2.default)('ncc:commandSession');

var VERSION = 1;
var COMMAND_URL = '/api/Command.nhn';

var CHAT_IMGS_URL = 'http://cafechat.phinf.naver.net';

var validateResponse = function validateResponse(body) {
  if (body.retCode === _config.COMMAND_RESULT_CODE.SUCCESS) {
    return body;
  }
  // Otherwise an error.
  console.log(body);
  var error = new Error(body.retTitle + ', ' + body.retMsg);
  error.retCode = body.retCode;
  throw error;
};

var CommandSession = (function (_RawSession) {
  _inherits(CommandSession, _RawSession);

  function CommandSession(server, credentials) {
    _classCallCheck(this, CommandSession);

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(CommandSession).call(this, server, credentials));

    _this.rooms = {};
    _this.cafes = {};
    _this.roomsLoaded = false;
    return _this;
  }

  _createClass(CommandSession, [{
    key: 'sendCommand',
    value: function sendCommand(command, body) {
      if (!this.connected) return Promise.reject(new Error('Not connected'));
      log('Sending command %s', command);
      return this.request({
        url: _config.CHAT_BROKER_SSL_URL + COMMAND_URL,
        method: 'POST',
        timeout: 10000,
        json: true,
        body: {
          ver: VERSION,
          uid: this.username,
          tid: +new Date(),
          sid: this.sid,
          deviceType: _rawSession.DEVICE_TYPE.WEB,
          cmd: _config.COMMAND_TYPE[command],
          bdy: body
        }
      });
    }
    // options: name, isPublic, is1to1.
    // However, creating non-1to1 room isn't supported now. Be careful.

  }, {
    key: 'createRoom',
    value: function createRoom(cafe, userList) {
      var _this2 = this;

      var options = arguments.length <= 2 || arguments[2] === undefined ? {
        name: '', isPublic: false
      } : arguments[2];
      var captcha = arguments.length <= 3 || arguments[3] === undefined ? {} : arguments[3];

      var users = userList.map(function (user) {
        return user.id || user;
      });
      return this.sendCommand('CreateRoom', {
        cafeId: cafe.id.toString(),
        roomType: users.length !== 2 || options.isPublic ? 1 : 0,
        openType: options.isPublic ? 'O' : 'C',
        masterUserId: options.masterId || this.username,
        roomName: options.name,
        captchaKey: captcha.key || '',
        captchaValue: captcha.value || '',
        memberList: users
      }).then(function (res) {
        if (res.bdy && res.bdy.roomId) {
          // Consider it done.
          var _res$bdy = res.bdy;
          var roomId = _res$bdy.roomId;
          var cafeId = _res$bdy.cafeId;

          return _this2.joinRoom(cafeId, roomId);
        }
        console.log(res);
      });
    }
  }, {
    key: 'inviteRoom',
    value: function inviteRoom(room, userList) {
      var captcha = arguments.length <= 2 || arguments[2] === undefined ? {} : arguments[2];

      var users = userList.map(function (user) {
        return user.id || user;
      });
      return this.sendCommand('InviteRoom', {
        cafeId: room.cafe.id.toString(),
        roomId: room.id,
        captchaKey: captcha.key || '',
        captchaValue: captcha.value || '',
        inviteeList: users
      }).then(validateResponse).then(function () {
        // Done
      }, function (res) {
        // .bdy.token.captchaKey
        // .bdy.token.imageUrl
        if (res.resCode === 1036 || res.resCode === 1037) {
          throw {
            captcha: true,
            key: res.bdy.token.captchaKey,
            image: res.bdy.token.imageUrl
          };
        }
      });
    }
    // This deletes room from the USER. which means, this doesn't terminate
    // the room.

  }, {
    key: 'deleteRoom',
    value: function deleteRoom(room) {
      var _this3 = this;

      // Check validity of the room data.
      if (this.roomsLoaded && this.rooms[room.id] == null) {
        return Promise.reject(new Error('Not joined in the room'));
      }
      return this.sendCommand('DeleteRoom', {
        cafeId: room.cafe.id,
        roomId: room.id
      }).then(validateResponse).then(function () {
        log('Successfully deleted room %s', room.name);
        // Remove room from the list, I suppose
        delete room.users[_this3.username];
        room.joined = false;
        room.userCount--;
        delete _this3.rooms[room.id];
        // But don't delete it from the cafe; Room doesn't get removed even if
        // the user leave the room... unless there is only one user in the room.
        if (room.userCount === 0) delete room.cafe.rooms[room.id];
        return room;
      }, function (body) {
        switch (body.retCode) {
          case _config.COMMAND_TYPE.NOT_FOUND_ROOM:
            throw new Error('DeleteRoom: Cannot find room ' + room.id);
          // Remove room anyway
          case _config.COMMAND_TYPE.NOT_ROOM_MEMBER:
            throw new Error('DeleteRoom: Not a member of room ' + room.id);
          // Still, remove room anyway
          default:
            throw body;
        }
      });
    }
    // This closes the room forcefully. Only staffs are able to do it.

  }, {
    key: 'closeOpenroom',
    value: function closeOpenroom(room) {
      var _this4 = this;

      // Check validity of the room data.
      if (this.roomsLoaded && room.cafe.rooms[room.id] == null) {
        return Promise.reject(new Error('Room does not exist'));
      }
      return this.sendCommand('CloseOpenroom', {
        cafeId: room.cafe.id,
        roomId: room.id
      }).then(validateResponse).then(function () {
        log('Successfully closed room %s', room.name);
        // The room has been terminated; Delete from cafe and room.
        room.joined = false;
        delete _this4.rooms[room.id];
        delete room.cafe.rooms[room.id];
        return room;
      }, function (body) {
        // TODO What's the error code of this?
        throw body;
      });
    }
    // Fetches data from the server and elevates loading level to 0
    // This also joins to the room if the user hasn't joined yet. Quite
    // weird, right?

  }, {
    key: 'syncRoom',
    value: function syncRoom(room) {
      var _this5 = this;

      room.loading = true;
      return this.sendCommand('SyncRoom', {
        cafeId: room.cafe.id,
        roomId: room.id,
        // TODO What does this mean?
        updateTimeSec: 0,
        // TODO It's 1 or 20
        size: 1
      }).then(validateResponse).then(function (command) {
        var body = command.bdy;
        var newRoom = (0, _translate.translateSyncRoom)(_this5, body);
        log('Synced room %s', newRoom.name);
        // Oh well doesn't matter. elevate loading level to 0
        newRoom.load = 0;
        newRoom.loading = false;
        return newRoom;
      }, function (body) {
        room.loading = false;
        // TODO What's the error code of this?
        throw body;
      });
    }
    // Just an alias..

  }, {
    key: 'joinRoom',
    value: function joinRoom(cafeId, roomId) {
      if (cafeId.id) return this.syncRoom(cafeId);
      return this.syncRoom({
        id: roomId,
        cafe: {
          id: cafeId
        }
      });
    }
    // Fetches connected chat room list

  }, {
    key: 'getRoomList',
    value: function getRoomList() {
      var _this6 = this;

      return this.sendCommand('GetRoomList', {
        // Just directly using room list protocol from original code
        cafeId: 0,
        lastMsgTimeSec: 0,
        // Type does nothing - :P
        type: 1,
        size: 100
      }).then(validateResponse).then(function (res) {
        _this6.roomsLoaded = true;
        var roomList = res.bdy.roomList;
        // We only need to look for roomList.

        var rooms = roomList.map(function (room) {
          return (0, _translate.translateRoomList)(_this6, room);
        });
        return rooms;
      }, function (body) {
        throw body;
      });
    }
    // Changes room name

  }, {
    key: 'changeRoomName',
    value: function changeRoomName(room, name) {
      var _this7 = this;

      if (room.master && room.master.id !== this.username) {
        return Promise.reject(new Error('You are not master of the room'));
      }
      return this.sendCommand('ChangeRoomName', {
        cafeId: room.cafe.id,
        roomId: room.id,
        roomName: name
      }).then(validateResponse).then(function () {
        // Since we don't know message state at this time, request message
        // sync.
        return _this7.syncMsg(room);
      });
    }
    // Hands room master; Can accept user ID or user object.

  }, {
    key: 'delegateMaster',
    value: function delegateMaster(room, user) {
      var _this8 = this;

      if (room.master && room.master.id !== this.username) {
        return Promise.reject(new Error('You are not master of the room'));
      }
      if (user.id) user = user.id;
      return this.sendCommand('DelegateMaster', {
        cafeId: room.cafe.id,
        roomId: room.id,
        targetMemberId: user
      }).then(validateResponse).then(function () {
        // Resync is required for this too.
        return _this8.syncMsg(room);
      });
    }
    // 'Bans' the user. Can accept user ID or user object.

  }, {
    key: 'rejectMember',
    value: function rejectMember(room, user) {
      var _this9 = this;

      if (room.master && room.master.id !== this.username) {
        return Promise.reject(new Error('You are not master of the room'));
      }
      if (user.id) user = user.id;
      return this.sendCommand('RejectMember', {
        cafeId: room.cafe.id,
        roomId: room.id,
        targetMemberId: user
      }).then(validateResponse).then(function () {
        // Resync is required for this too.
        return _this9.syncMsg(room);
      });
    }
    // Returns a list of cafe open rooms

  }, {
    key: 'findOpenRoomList',
    value: function findOpenRoomList(cafe) {
      var _this10 = this;

      var order = arguments.length <= 1 || arguments[1] === undefined ? 'time' : arguments[1];
      var page = arguments.length <= 2 || arguments[2] === undefined ? 1 : arguments[2];

      return this.sendCommand('FindOpenRoomList', {
        cafeId: cafe.id,
        orderBy: order === 'time' ? 'LastMsgTimestamp' : 'RoomName',
        page: page
      }).then(validateResponse).then(function (res) {
        var openRoomList = res.bdy.openRoomList;

        var rooms = openRoomList.map(function (room) {
          return (0, _translate.translateRoomFromFind)(_this10, room);
        });
        console.log(rooms);
        return rooms;
      });
    }
    // Returns a list of joined cafes

  }, {
    key: 'findMyCafeList',
    value: function findMyCafeList() {
      var _this11 = this;

      return this.sendCommand('FindMyCafeList', {}).then(validateResponse).then(function (res) {
        var myCafeList = res.bdy.myCafeList;

        var cafes = myCafeList.map(function (cafe) {
          return (0, _translate.translateCafeFromFind)(_this11, cafe);
        });
        console.log(cafes);
        return cafes;
      });
    }
    // Syncs lost message from the server

  }, {
    key: 'syncMsg',
    value: function syncMsg(room) {
      var _this12 = this;

      if (room.sync) {
        room.sync = false;
        // TODO move this somewhere else?
        return this.sendCommand('SyncMsg', {
          cafeId: room.cafe.id,
          roomId: room.id,
          lastMsgSn: room.lastMsgSn,
          size: 100
        }).then(validateResponse).then(function (res) {
          log('Synced messages; digesting');
          var body = res.bdy;
          // Digest missed messages
          room.lastMsgSn = body.lastMsgSn;
          room.sync = true;
          var list = body.msgList.map(function (newMessage) {
            // Fill unsent information
            newMessage.cafeId = room.cafe.id;
            newMessage.roomId = room.id;
            // There's no way to obtain this information if unsync has occurred
            newMessage.msgId = null;
            return _this12.handleMessage(newMessage);
          });
          log('Digestion done');
          return list;
        }).catch(function (err) {
          console.log(err);
        });
      }
      return Promise.resolve();
    }
    // Sends a message to the server

  }, {
    key: 'sendMsg',
    value: function sendMsg(message) {
      var _this13 = this;

      var rawMsg = {
        cafeId: message.room.cafe.id,
        roomId: message.room.id,
        msgType: _message.MSG_TYPE[message.type],
        // This doesn't mean anything! What's this for?
        msgId: new Date().valueOf(),
        msg: message.message
      };
      return this.sendCommand('SendMsg', rawMsg).then(validateResponse).then(function (command) {
        var body = command.bdy;
        // Combine rawMsg with body
        Object.assign(rawMsg, body, {
          // 'sent' flag shows that this is sent by itself so
          // bots and other things shouldn't process it
          sent: true,
          // Also, inject username simply because it's not sent from the server
          senderId: _this13.username
        });
        // TODO restore information if possible
        if (message.type === 'sticker') {
          rawMsg.msg = JSON.stringify({
            stickerId: rawMsg.msg
          });
        } else if (message.type === 'image') {
          rawMsg.msg = JSON.stringify(Object.assign({}, message.message, {
            orgUrl: CHAT_IMGS_URL + message.message.path
          }));
        }
        log('Sent message; handling');
        // Then, handle it
        return _this13.handleMessage(rawMsg);
      }, function (body) {
        // TODO What's the error code of this?
        throw body;
      });
    }
    // Get old messages and returns them.

  }, {
    key: 'getMsg',
    value: function getMsg(room, start, end) {
      var _this14 = this;

      return this.sendCommand('GetMsg', {
        cafeId: room.cafe.id,
        roomId: room.id,
        startMsgSn: start,
        endMsgSn: end
      }).then(validateResponse).then(function (res) {
        var msgList = res.bdy.msgList;
        // However, we need to mark that this message is 'old' to prevent
        // changing room information by translator.

        var messages = msgList.map(function (msg) {
          return (0, _translate.translateMessage)(_this14, Object.assign({}, msg, {
            old: true,
            roomId: room.id,
            cafeId: room.cafe.id
          }));
        });
        return messages;
      });
    }
    // Send an acknowledge to the server. However, this is only necessary if
    // you're making an actual chat client, it's not required for bots.
    // However, node-ncc-es6 doesn't handle lastAckSn yet, so this isn't likely
    // to be used anyway. TODO feel free to send a pull request.

  }, {
    key: 'ackMsg',
    value: function ackMsg(message) {
      return this.sendCommand('AckMsg', {
        cafeId: message.room.cafe.id,
        roomId: message.room.id,
        lastMsgSn: message.id
      })
      // However, validation isn't required.
      .then(validateResponse);
    }
    // While commandSession itself doesn't use polling, but we still need to
    // process message. Of course it's very raw and not ready to use.
    // Session will extend it to make it better.

  }, {
    key: 'handleMessage',
    value: function handleMessage(message) {
      // Emit an event...
      this.emit('rawMessage', message);
    }
  }]);

  return CommandSession;
})(_rawSession2.default);

exports.default = CommandSession;