'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RESULT_CODE = exports.DEVICE_TYPE = exports.TIME_CONFIG = exports.CONNECT_URL = exports.POLL_URL = undefined;

var _requestPromise = require('request-promise');

var _requestPromise2 = _interopRequireDefault(_requestPromise);

var _config = require('./config.js');

var _events = require('events');

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var log = (0, _debug2.default)('ncc:rawSession');

// Server API endpoint
// There's 'close' too, but I don't think I'll need it.
// If anyone needs it, feel free to open an issue
var POLL_URL = exports.POLL_URL = '/poll.nhn';
var CONNECT_URL = exports.CONNECT_URL = '/conn.nhn';

// Time related config
var TIME_CONFIG = exports.TIME_CONFIG = {
  POLLING_TIMEOUT: 20,
  CONN_TIMEOUT: 3,
  POLL_RETRY_LIMIT_CNT: 3,
  CONN_RETRY_LIMIT_CNT: 10,
  POLL_SLEEP_DELAY: 0,
  CONN_SLEEP_DELAY: 500,
  CONN_SLEEP_MAX_DELAY: 3000
};

// Device type enums
var DEVICE_TYPE = exports.DEVICE_TYPE = {
  WEB: 2001
};

var RESULT_CODE = exports.RESULT_CODE = {
  HTTP_SUCCESS: 200,
  CMD_SUCCESS: 0,
  POLLING_RE_CONN: 204,
  ERR_INTERNAL_ERROR: 102,
  ERR_INVALID_PARAMETER: 105,
  ERR_INVALID_SESSION: 201,
  ERR_SESSION_NOT_FOUND: 202,
  ERR_SESSION_CONFLICT: 203,
  ERR_EXPIRED_COOKIE: 302,
  CONN_RESP: 10100
};

function getCallbackFn() {
  // Mimic Jindo's behaviour
  return 'window.__jindo2_callback._' + Math.floor(Math.random() * 900 + 100);
}

// Extract from JSONP data
function extractResponse(body) {
  // Or we could use regular expression..
  var unpacked = body.slice(body.indexOf('"') + 1, body.lastIndexOf('"'));
  // This breaks Korean text... Not sure why though.
  var decrypted = new Buffer(unpacked, 'base64').toString('utf8');
  return JSON.parse(decrypted);
}

// validate response...
var validateResponse = function validateResponse(message) {
  // Error will be thrown by request-promise if status code is not 200
  // This means we don't have to check status code (We can't check it anyway)
  if (message.retCode !== RESULT_CODE.CMD_SUCCESS) {
    throw message;
  }
  return message.bdy;
};

/**
 * Low-level session object to communicate with the chat server
 */

var RawSession = (function (_EventEmitter) {
  _inherits(RawSession, _EventEmitter);

  function RawSession(server, credentials) {
    _classCallCheck(this, RawSession);

    /**
     * The chat server's URL address. Should not end with '/'.
     * @type {String}
     */

    var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(RawSession).call(this));

    _this.server = server;
    /**
     * The credentials information used to connect to the server.
     * @type {Credentials}
     */
    _this.credentials = credentials;
    _this.username = null;
    _this.sid = null;
    // And tons of internal variables
    _this.connected = false;
    return _this;
  }

  _createClass(RawSession, [{
    key: 'connect',
    value: function connect() {
      var _this2 = this;

      var retries = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];
      var err = arguments[1];

      if (retries >= TIME_CONFIG.CONN_RETRY_LIMIT_CNT) {
        // Connection failed. Tata!
        if (err.stack) {
          this.emit('error', err);
          throw err;
        }
        if (err.retMsg) {
          this.emit('error', new Error(err.retMsg));
          throw new Error(err.retMsg);
        }
        this.emit('error', new Error(err));
        throw new Error(err);
      }
      if (this.connected) return Promise.reject('Already connected');
      log('Connecting to the session server');
      // Recreate request object here because of cookieJar object
      this.request = _requestPromise2.default.defaults({
        jar: this.credentials.cookieJar,
        // TODO test connection without it?
        strictSSL: false,
        headers: {
          'Content-Type': 'application/json; charset=UTF-8',
          'Referer': _config.CHAT_HOME_URL
        },
        timeout: TIME_CONFIG.POLLING_TIMEOUT * 1000
      });
      return this.request({
        url: this.server + CONNECT_URL,
        qs: {
          'callback_fn': getCallbackFn(),
          uid: this.credentials.username,
          tid: +new Date(),
          devType: DEVICE_TYPE.WEB,
          crypto: false
        }
      }).then(extractResponse).then(validateResponse).then(function (body) {
        log('Connected as session id %s', body.sid);
        // Why the server is sending AES encryption information? ...
        // Anyway, we've got the session id at this point if we don't have
        // an error.
        _this2.sid = body.sid;
        _this2.username = _this2.credentials.username;
        _this2.connected = true;
        _this2.emit('connect');
        // Start polling
        return _this2.schedulePoll();
      }, function (message) {
        log('Failed to connect to the session server');
        console.log(message);
        // Strangely, server sends 'invalid parameter' if cookie is missing
        if (message.retCode === RESULT_CODE.ERR_EXPIRED_COOKIE || message.retCode === RESULT_CODE.ERR_INVALID_PARAMETER) {
          // Relogin is required :/
          return _this2.credentials.login().then(function () {
            return _this2.connect(retries + 1, message);
          });
        }
        // Retry login
        // TODO wait for retry
        return _this2.connect(retries + 1, message);
      });
    }
  }, {
    key: 'disconnect',
    value: function disconnect() {
      log('Disconnecting from the session server');
      // Do nothing! :P
      this.connected = false;
      this.emit('disconnect');
    }
  }, {
    key: 'schedulePoll',
    value: function schedulePoll() {
      var retries = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];

      if (retries >= TIME_CONFIG.POLL_RETRY_LIMIT_CNT) {
        this.disconnect();
        return this.connect();
      }
      log('Scheduling polling');
      // I think 1000ms is too much... Well whatever.
      // TODO is cutting Promise chain really necessary?
      setTimeout(this.poll.bind(this, retries), TIME_CONFIG.POLL_SLEEP_DELAY);
    }
    // Only one poll cycle should be running in a single time

  }, {
    key: 'poll',
    value: function poll() {
      var _this3 = this;

      var retries = arguments.length <= 0 || arguments[0] === undefined ? 0 : arguments[0];

      if (!this.connected) return Promise.reject();
      log('Polling data');
      return this.request({
        url: this.server + POLL_URL,
        qs: {
          'callback_fn': getCallbackFn(),
          sid: this.sid,
          tid: +new Date(),
          crypto: false
        }
      }).then(extractResponse).then(function (message) {
        log('Polled data');
        var retCode = message.retCode;
        var bdy = message.bdy;

        if (retCode === RESULT_CODE.CMD_SUCCESS || retCode === RESULT_CODE.POLLING_RE_CONN) {
          if (retCode === RESULT_CODE.CMD_SUCCESS) {
            // Something was received
            _this3.handlePoll(bdy);
          }
          _this3.schedulePoll();
        } else {
          log('Polling data failed');
          console.log(message);
          switch (retCode) {
            case RESULT_CODE.ERR_INVALID_SESSION:
            case RESULT_CODE.ERR_SESSION_NOT_FOUND:
            case RESULT_CODE.ERR_SESSION_CONFLICT:
              _this3.disconnect();
              return _this3.connect();
            case RESULT_CODE.ERR_EXPIRED_COOKIE:
              _this3.disconnect();
              // Relogin is required :/
              return _this3.credentials.login().then(function () {
                return _this3.connect();
              });
            default:
              // Check limit?
              _this3.emit('error', message.retMsg);
              return _this3.schedulePoll(retries + 1);
          }
        }
      }).catch(function (err) {
        _this3.emit('error', err);
        return _this3.schedulePoll(retries + 1);
      });
    }
  }, {
    key: 'handlePoll',
    value: function handlePoll(data) {
      // Emit an event...
      this.emit('data', data);
    }
  }]);

  return RawSession;
})(_events.EventEmitter);

exports.default = RawSession;