'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _commandSession = require('./commandSession.js');

var _commandSession2 = _interopRequireDefault(_commandSession);

var _config = require('./config.js');

var _translate = require('./translate.js');

var _uploadImage = require('./uploadImage.js');

var _uploadImage2 = _interopRequireDefault(_uploadImage);

var _debug = require('debug');

var _debug2 = _interopRequireDefault(_debug);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var log = (0, _debug2.default)('ncc:session');

var NOTI_TYPE = {
  Msg: 93001,
  Invited: 93002,
  ChangeRoomName: 93003,
  DeleteRoom: 93004,
  DelegateMaster: 93005,
  JoinRoom: 93006,
  RejectMember: 93007,
  ClosedOpenroom: 93008
};

var Session = (function (_CommandSession) {
  _inherits(Session, _CommandSession);

  function Session(credentials) {
    _classCallCheck(this, Session);

    // Randomly choose a server
    var server = _config.SESSION_SERVER_URLS[Math.floor(Math.random() * 10) + 11];
    return _possibleConstructorReturn(this, Object.getPrototypeOf(Session).call(this, server, credentials));
  }

  _createClass(Session, [{
    key: 'handlePoll',
    value: function handlePoll(data) {
      var _this2 = this;

      data.forEach(function (item) {
        _this2.handleNotification(item);
      });
    }
  }, {
    key: 'handleNotification',
    value: function handleNotification(item) {
      log('Handling notifcation %s', item.cmd);
      switch (item.cmd) {
        case NOTI_TYPE.Msg:
          // Handle message
          this.handleMessage(item.bdy);
          break;
        case NOTI_TYPE.ClosedOpenroom:
          // Room has closed; Remove reference to it
          delete this.rooms[item.bdy.roomId];
          break;
        case NOTI_TYPE.Invited:
        case NOTI_TYPE.ChangeRoomName:
        case NOTI_TYPE.DeleteRoom:
        case NOTI_TYPE.DelegateMaster:
        case NOTI_TYPE.JoinRoom:
        case NOTI_TYPE.RejectMember:
        default:
          // Create chatroom / cafe if it doesn't exist.
          if (this.rooms[item.bdy.roomId] == null) {
            // TODO There's more data in it, but I'll leave like this now.
            this.rooms[item.bdy.roomId] = (0, _translate.translateRoomFromMessage)(this, item.bdy);
            log('Creating missing room');
          }
          var room = this.rooms[item.bdy.roomId];
          // Just resync it; Heh.
          this.syncMsg(room);
          break;
      }
    }
  }, {
    key: 'handleMessage',
    value: function handleMessage(message) {
      // Create chatroom / cafe if it doesn't exist.
      if (this.rooms[message.roomId] == null) {
        // TODO There's more data in it, but I'll leave like this now.
        this.rooms[message.roomId] = (0, _translate.translateRoomFromMessage)(this, message);
        log('Creating missing room');
      }
      var room = this.rooms[message.roomId];
      // Ignore if room is not in sync.
      if (!room.sync) return;
      // Handle 'sent by itself' messages specially
      // Server doesn't have this data, so we'll lost this data if we sync it
      // again.
      if (message.sent) {
        log('Processing sent message');
        room.lastSentMsgSn = message.msgSn;
      }
      // Check if message is in sync. If we have missing messages,
      // Drop current message and request new one.
      if (message.msgSn - room.lastMsgSn > 1) {
        log('Sync failed; requesting resync');
        return this.syncMsg(room);
      }
      room.lastMsgSn = message.msgSn;
      var newMessage = (0, _translate.translateMessage)(this, message);
      // Handle 'sent by itself' message.
      if (newMessage.id === room.lastSentMsgSn) {
        log('Handling sent message');
        newMessage.sent = true;
      }
      // Handle force resync messages.
      if (newMessage.resync) {
        // However, we don't need wait for result to come. I think so?
        this.syncRoom(newMessage.room);
      }
      room.lastMessage = newMessage;
      this.emit('message', newMessage);
      return newMessage;
    }
    // Helper functions to send messages

  }, {
    key: 'sendText',
    value: function sendText(room, text) {
      return this.sendMsg({
        room: room,
        type: 'text',
        message: text + ''
      });
    }
  }, {
    key: 'sendSticker',
    value: function sendSticker(room, stickerId) {
      return this.sendMsg({
        room: room,
        type: 'sticker',
        message: stickerId
      });
    }
    // Image can be either stream, or uploaded image object

  }, {
    key: 'sendImage',
    value: function sendImage(room, image, options) {
      if (image.path == null || image.fileSize == null) {
        // Do upload first
        return (0, _uploadImage2.default)(this.request, image, options)
        // Then recall this routine
        .then(this.sendImage.bind(this, room));
      }
      return this.sendMsg({
        room: room,
        type: 'image',
        message: image
      });
    }
  }]);

  return Session;
})(_commandSession2.default);

exports.default = Session;